@@ -1352,7 +1352,6 @@ def test_append(s3):
         ServerSideEncryption="AES256",
         StorageClass="REDUCED_REDUNDANCY",
         WebsiteRedirectLocation="https://www.example.com/",
-        BucketKeyEnabled=False,
     )
     with s3.open(a, "wb", **head) as f:
         f.write(b"data")
@@ -1509,8 +1508,11 @@ def test_tags(s3):
     assert s3.get_tags(fname) == tagset
 
 
-def test_versions(s3):
-    versioned_file = versioned_bucket_name + "/versioned_file"
+@pytest.mark.parametrize("prefix", ["", "/dir", "/dir/subdir"])
+def test_versions(s3, prefix):
+    parent = versioned_bucket_name + prefix
+    versioned_file = parent + "/versioned_file"
+
     s3 = S3FileSystem(
         anon=False, version_aware=True, client_kwargs={"endpoint_url": endpoint_uri}
     )
@@ -1538,6 +1540,17 @@ def test_versions(s3):
         assert fo.version_id == first_version
         assert fo.read() == b"1"
 
+    versioned_file_v1 = f"{versioned_file}?versionId={first_version}"
+    versioned_file_v2 = f"{versioned_file}?versionId={second_version}"
+
+    assert s3.ls(parent) == [versioned_file]
+    assert set(s3.ls(parent, versions=True)) == {versioned_file_v1, versioned_file_v2}
+
+    assert s3.exists(versioned_file_v1)
+    assert s3.info(versioned_file_v1)
+    assert s3.exists(versioned_file_v2)
+    assert s3.info(versioned_file_v2)
+
 
 def test_list_versions_many(s3):
     # moto doesn't actually behave in the same way that s3 does here so this doesn't test
@@ -2615,6 +2628,8 @@ def test_async_stream(s3_base):
         f = await fs.open_async(fn, mode="rb", block_seze=1000)
         while True:
             got = await f.read(1000)
+            assert f.size == len(data)
+            assert f.tell()
             if not got:
                 break
             out.append(got)
